/*****************************************************************************/
//
//   (c) Copyright 2005 - 2009
//   National Instruments Corporation.
//   All rights reserved.
//
//   This is the C/C++ header file for the ECU Measurement and
//   Calibration Toolkit Application Programming Interface (API).
//
//   This header file is not intended for use as a programming reference.
//   For complete information on the data types and functions
//   declared in this file, refer to the ECU Measurement and
//   Calibration Toolkit manual.
//
/*****************************************************************************/


#ifndef __niemc_h__
#define __niemc_h__


#if defined(LINUX_BUILD)
   #define VX_EXPORT_SYMBOL __attribute__ ((section (".export")))
   #define MS_EXPORT_SYMBOL
   #define CALLING_CONVENTION
#else
   #define VX_EXPORT_SYMBOL
   #define MS_EXPORT_SYMBOL __declspec(dllexport)
   #define CALLING_CONVENTION __cdecl
#endif


#if defined(_CVI_)
   #pragma EnableLibraryRuntimeChecking
#endif

#ifndef NIEMC_TYPES_DEFINED
   #ifndef _NI_i8_DEFINED_
      #define _NI_i8_DEFINED_
      typedef signed char     i8;
   #endif
   #ifndef _NI_i16_DEFINED_
      #define _NI_i16_DEFINED_
      typedef signed short    i16;
   #endif
   #ifndef _NI_i32_DEFINED_
      #define _NI_i32_DEFINED_
      typedef signed int      i32;
   #endif
   #ifndef _NI_u8_DEFINED_
      #define _NI_u8_DEFINED_
      typedef unsigned char   u8;
   #endif
   #ifndef _NI_u16_DEFINED_
      #define _NI_u16_DEFINED_
      typedef unsigned short  u16;
   #endif
   #ifndef _NI_u32_DEFINED_
      #define _NI_u32_DEFINED_
      typedef unsigned int    u32;
   #endif
   #ifndef _NI_i64_DEFINED
      #define _NI_i64_DEFINED
      #if (defined(_MSC_VER) || defined(_CVI_) || defined(__BORLANDC__))
         typedef __int64     i64;
      #elif defined(__GNUC__)
         typedef long long int   i64;
      #endif
   #endif
   #ifndef _NI_u64_DEFINED
      #define _NI_u64_DEFINED
      #if (defined(_MSC_VER) || defined(_CVI_) || defined(__BORLANDC__))
         typedef unsigned __int64     u64;
      #elif defined(__GNUC__)
         typedef unsigned long long int   u64;
      #endif
   #endif
   #ifndef _NI_f32_DEFINED_
      #define _NI_f32_DEFINED_
      typedef float           f32;
   #endif
   #ifndef _NI_f64_DEFINED_
      #define _NI_f64_DEFINED_
      typedef double          f64;
   #endif
   typedef void* mcVoidPtr;
#endif // NIEMC_TYPES_DEFINED

#ifndef _NI_str_DEFINED_
   #define _NI_str_DEFINED_
   typedef char *          str;
#endif
#ifndef _NI_cstr_DEFINED_
   #define _NI_cstr_DEFINED_
   typedef const char *    cstr;
#endif

typedef struct { u32 LowPart; u32 HighPart;} mcTypeTimestamp;

typedef enum
{
   eCommApiUnknown   = 0,     ///< Communication API is not yet determined
   eCommApiFrame     = 1,     ///< Communication is based on NI-CAN frame API
   eCommApiChannel   = 2      ///< Communication is based on NI-CAN channel API
} mcTypeCommApi;

#define WAIT_ALLOC                  100
#define WAIT_USAGE                  100
#define WAIT_STARTSTOP              10000
#define WAIT_CALLBACK               100

// Constants used to query the names of the
// ECUs, measurements, characteristics or event channels from the A2L file.
#define  mcTypeECUNames                   0           ///< Type for ECU names
#define  mcTypeMeasurementNames           1           ///< Type for measurement names
#define  mcTypeCharacteristicNames        2           ///< Type for characteristic names
#define  mcTypeEventChannelNames          3           ///< Type for event channel names
#define  mcTypeDefinedPagesNames          4           ///< Type for defined pages names
#define  mcTypeGroupNames                 5           ///< Type for asking GROUP names from the ECU
#define  mcTypeGroup_SubGroupNames        6           ///< Type for asking SUB_GROUP names from the GROUP
#define  mcTypeGroup_MeasurementNames     7           ///< Type for asking MEASUREMENT names from the GROUP
#define  mcTypeGroup_CharacteristicNames  8           ///< Type for asking CHARACTERISTIC names from the GROUP
#define  mcTypeFuncNames                  9           ///< Type for asking FUNCTION names from the ECU
#define  mcTypeFunc_DefCharacNames            10           ///< Type for asking DEF_CHARACTERISTIC names from the FUNCTION
#define  mcTypeFunc_RefCharacNames            11           ///< Type for asking REF_CHARACTERISTIC names from the FUNCTION
#define  mcTypeFunc_InMeasNames               12           ///< Type for asking IN_MEASUREMENT names from the FUNCTION
#define  mcTypeFunc_OutMeasNames              13           ///< Type for asking OUT_MEASUREMENT names from the FUNCTION
#define  mcTypeFunc_LocMeasNames              14           ///< Type for asking LOC_MEASUREMENT names from the FUNCTION
#define  mcTypeFunc_SubFuncNames              15           ///< Type for asking SUB_FUNCTION names from the FUNCTION
#define  mcTypeGroup_FunctionListNames        16           ///< Type for asking FUNCTION_LIST names from the GROUP
#define  mcTypeTLI_XcpOnCanNames              17           ///< Type for asking TRANSPORT_LAYER_INSTANCE names from XCP_ON_CAN in XCPplus
#define  mcTypeTLI_XcpOnTcpNames              18           ///< Type for asking TRANSPORT_LAYER_INSTANCE names from XCP_ON_TCP_IP in XCPplus
#define  mcTypeTLI_XcpOnUdpNames              19           ///< Type for asking TRANSPORT_LAYER_INSTANCE names from XCP_ON_UDP_IP in XCPplus

// Refnum definitions
#define  mcRefNumType                     0xF0000000  ///< internal to extract the refnum type from the refnum
#define  mcDBRefNum                       0x10000000  ///< indicator for database refnum
#define  mcECURefNum                      0x20000000  ///< indicator for ECU refnum
#define  mcDAQRefNum                      0x40000000  ///< indicator for DAQ refnum

// Status code definitions
#define  mcStatusOK                                 0    ///< everything was fine
#define  mc_WARNING_BASE      (mcTypeStatus)(  301000 )  ///< base for mc warnings
#define  mc_ERROR_BASE        (mcTypeStatus)( -301000 )  ///< base for mc errors

// Warnings mc_WARNING_BASE         ... mc_WARNING_BASE +  59  ECU Measurement and Calibration Toolkit warnings
// Warnings mc_WARNING_BASE +  60   ... mc_WARNING_BASE + 109  CCP warnings
// Warnings mc_WARNING_BASE + 110   ... mc_WARNING_BASE + 129  A2L file reader warnings
#define  mcWarningBufferTooSmall                ( mc_WARNING_BASE +   0 )
#define  mcWarningECUinUse                      ( mc_WARNING_BASE +   1 )
#define  mcWarningPropertyIsNotInitialized      ( mc_WARNING_BASE +   2 )
#define  mcWarningScalingNotSupported           ( mc_WARNING_BASE +   3 )
#define  mcWarningDataOutOfRange                ( mc_WARNING_BASE +   4 )
#define  mcWarningOldData                       ( mc_WARNING_BASE +   5 )
#define  mcWarningScalingInverseNotDefined      ( mc_WARNING_BASE +   6 )
#define  mcWarningScalingTextResult             ( mc_WARNING_BASE +   7 )
#define  mcWarningECUCommunication              ( mc_WARNING_BASE +   8 )
#define  mcWarningEventChannelIgnored           ( mc_WARNING_BASE +   9 )
#define  mcWarningECUResponseLen                ( mc_WARNING_BASE +  10 )
#define  mcWarningUnsupportedFeature            ( mc_WARNING_BASE +  23 )
#define  mcWarningTrialVersion                  ( mc_WARNING_BASE +  29 )
#define  mcWarningFunctionNotDefinedForProtocol ( mc_WARNING_BASE +  42 )

#define  mcWarningXCPCommandTooLong             ( mc_WARNING_BASE +  60 )
#define  mcWarningXCPCommandResponseTooLong     ( mc_WARNING_BASE +  61 )

#define  mcWarningUnsupportedCmd                ( mc_WARNING_BASE + 132 )

// Errors mc_ERROR_BASE         ... mc_ERROR_BASE +  59  ECU Measurement and Calibration Toolkit errors
// Errors mc_ERROR_BASE +  60   ... mc_ERROR_BASE + 109  CCP errors
// Errors mc_ERROR_BASE + 110   ... mc_ERROR_BASE + 129  A2L file reader errors
#define  mcErrorChecksumFileNotFound            ( mc_ERROR_BASE   -   0 )
#define  mcErrorInvalidDBRef                    ( mc_ERROR_BASE   -   1 )
#define  mcErrorInvalidECURef                   ( mc_ERROR_BASE   -   2 )
#define  mcErrorInvalidDAQRef                   ( mc_ERROR_BASE   -   3 )
#define  mcErrorTooManyOpenFiles                ( mc_ERROR_BASE   -   4 )
#define  mcErrorInvalidRefNum                   ( mc_ERROR_BASE   -   5 )
#define  mcErrorInternal                        ( mc_ERROR_BASE   -   6 )
#define  mcErrorCommParamsNotInitialized        ( mc_ERROR_BASE   -   7 )
#define  mcErrorInvalidPropertyID               ( mc_ERROR_BASE   -   8 )
#define  mcErrorInvalidPropertyValueSize        ( mc_ERROR_BASE   -   9 )
#define  mcErrorPropertyIsReadOnly              ( mc_ERROR_BASE   -  10 )
#define  mcErrorMAXConnectionsExceeded          ( mc_ERROR_BASE   -  11 )
#define  mcErrorInvalidBitMask                  ( mc_ERROR_BASE   -  12 )
#define  mcErrorInvalidMeasurementName          ( mc_ERROR_BASE   -  13 )
#define  mcErrorMAXDAQTasksExceeded             ( mc_ERROR_BASE   -  14 )
#define  mcErrorInvalidNumberOfSamples          ( mc_ERROR_BASE   -  15 )
#define  mcErrorInvalidParameter                ( mc_ERROR_BASE   -  16 )
#define  mcErrorNoMoreDAQListsAvailable         ( mc_ERROR_BASE   -  17 )
#define  mcErrorIndexOutOfRange                 ( mc_ERROR_BASE   -  18 )
#define  mcErrorInvalidBufferSize               ( mc_ERROR_BASE   -  19 )
#define  mcErrorInvalidDataSize                 ( mc_ERROR_BASE   -  20 )
#define  mcErrorSeedKeyLibNotFound              ( mc_ERROR_BASE   -  21 )
#define  mcErrorInvalidPropertyValue            ( mc_ERROR_BASE   -  22 )
#define  mcErrorUnsupportedFeature              ( mc_ERROR_BASE   -  23 )
#define  mcErrorIncorrectDimension              ( mc_ERROR_BASE   -  24 )
#define  mcErrorInvalidByteOrder                ( mc_ERROR_BASE   -  25 )
#define  mcErrorUnsupportedDataFormat           ( mc_ERROR_BASE   -  26 )
#define  mcErrorDLLnotFound                     ( mc_ERROR_BASE   -  27 )
#define  mcErrorFunctionNotFound                ( mc_ERROR_BASE   -  28 )
#define  mcErrorNoLicense                       ( mc_ERROR_BASE   -  29 )
#define  mcErrorAxisNotDefined                  ( mc_ERROR_BASE   -  30 )
#define  mcErrorReadOnly                        ( mc_ERROR_BASE   -  31 )
#define  mcErrorECUNotDefined                   ( mc_ERROR_BASE   -  32 )
#define  mcErrorCharacteristicNotDefined        ( mc_ERROR_BASE   -  33 )
#define  mcErrorMeasurementNotDefined           ( mc_ERROR_BASE   -  34 )
#define  mcErrorScalingNotDefined               ( mc_ERROR_BASE   -  35 )
#define  mcErrorRecordLayoutNotDefined          ( mc_ERROR_BASE   -  36 )
#define  mcErrorVirtualMeasurementsNotSupported ( mc_ERROR_BASE   -  37 )
#define  mcErrorExceptionInExternalCode         ( mc_ERROR_BASE   -  38 )
#define  mcErrorSeedKeyFunctionNotFound         ( mc_ERROR_BASE   -  39 )
#define  mcErrorChecksumFunctionNotFound        ( mc_ERROR_BASE   -  40 )
#define  mcErrorInternalTimeout                 ( mc_ERROR_BASE   -  41 )
#define  mcErrorFunctionNotDefinedForProtocol   ( mc_ERROR_BASE   -  42 )
#define  mcErrorBufferedOnlyWithTimestamp       ( mc_ERROR_BASE   -  43 )
#define  mcErrorUnknownChecksumType             ( mc_ERROR_BASE   -  44 )
#define  mcErrorIncorrectMode                   ( mc_ERROR_BASE   -  45 )
#define  mcErrorUnsupportedProtocol             ( mc_ERROR_BASE   -  46 )
#define  mcErrorIP                              ( mc_ERROR_BASE   -  47 )
#define  mcErrorLibraryNotLoaded                ( mc_ERROR_BASE   -  48 )
#define  mcErrorListFull                        ( mc_ERROR_BASE   -  49 )
#define  mcErrorParamOutOfRange                 ( mc_ERROR_BASE   -  50 )
#define  XCPErrGeneric                          ( mc_ERROR_BASE   -  51 )
#define  XCPErrVerify                           ( mc_ERROR_BASE   -  52 )
#define  mcErrorIPConnection                    ( mc_ERROR_BASE   -  53 )
#define  mcErrorGranularityFit                  ( mc_ERROR_BASE   -  54 )
#define  mcErrorInvalidEventChannel             ( mc_ERROR_BASE   -  55 )
#define  mcErrorInsufficientLicense             ( mc_ERROR_BASE   -  56 )
#define  mcErrorUnsupportedProperty             ( mc_ERROR_BASE   -  57 )
#define  mcErrorTimeout                         ( mc_ERROR_BASE   -  58 )
#define  mcErrorObjectAlreadyExists             ( mc_ERROR_BASE   -  59 )

#define  CCPStatusOK                                                  0
#define  CCPERROR                               ( mc_ERROR_BASE   -  60 )
#define  CCPErrDAQOverload                      ( mc_ERROR_BASE   -  60 )

#define  CCPErrBusy                             ( mc_ERROR_BASE   -  61 )
#define  CCPErrDAQBusy                          ( mc_ERROR_BASE   -  62 )
#define  CCPErrTimeout                          ( mc_ERROR_BASE   -  63 )

#define  CCPErrKeyRequest                       ( mc_ERROR_BASE   -  64 )
#define  CCPErrSessionStatusRequest             ( mc_ERROR_BASE   -  65 )

#define  CCPErrColdStartRequest                 ( mc_ERROR_BASE   -  66 )
#define  CCPErrCalDataInitRequest               ( mc_ERROR_BASE   -  67 )
#define  CCPErrDAQListInitRequest               ( mc_ERROR_BASE   -  68 )
#define  CCPErrCodeUpdateRequest                ( mc_ERROR_BASE   -  69 )

#define  CCPErrUnknownCommand                   ( mc_ERROR_BASE   -  70 )
#define  CCPErrCommandSyntax                    ( mc_ERROR_BASE   -  71 )
#define  CCPErrParameterOutOfRange              ( mc_ERROR_BASE   -  72 )
#define  CCPErrAccessDenied                     ( mc_ERROR_BASE   -  73 )
#define  CCPErrOverload                         ( mc_ERROR_BASE   -  74 )
#define  CCPErrAccessLocked                     ( mc_ERROR_BASE   -  75 )
#define  CCPErrResourceNotAvailable             ( mc_ERROR_BASE   -  76 )
#define  CCPErrInvalidErrorCode                 ( mc_ERROR_BASE   -  77 )

#define  CCPErrInvalidFrame                     ( mc_ERROR_BASE   -  78 )
#define  CCPErrCROOpen                          ( mc_ERROR_BASE   -  79 )
#define  CCPErrCRMOpen                          ( mc_ERROR_BASE   -  80 )
#define  CCPErrCCPSlaveError                    ( mc_ERROR_BASE   -  81 )
#define  CCPErrCCPCommandCounter                ( mc_ERROR_BASE   -  82 )

#define  CCPErrInvalidHandle                    ( mc_ERROR_BASE   -  83 )
#define  CCPErrInvalidODTNo                     ( mc_ERROR_BASE   -  84 )
#define  CCPErrInvalidODTEntry                  ( mc_ERROR_BASE   -  85 )
#define  CCPErrInvalidODTEntrySize              ( mc_ERROR_BASE   -  86 )
#define  CCPErrInvalidDAQListNo                 ( mc_ERROR_BASE   -  87 )
#define  CCPErrInvalidDeviceNo                  ( mc_ERROR_BASE   -  88 )
#define  CCPErrInvalidDataFormat                ( mc_ERROR_BASE   -  89 )
#define  CCPErrSignalNotDefined                 ( mc_ERROR_BASE   -  90 )
#define  CCPErrDuplicateSignalDefinition        ( mc_ERROR_BASE   -  91 )
#define  CCPErrInvalidConnection                ( mc_ERROR_BASE   -  92 )
#define  CCPErrInvalidVersion                   ( mc_ERROR_BASE   -  93 )

#define  CCPErrNullPointer                      ( mc_ERROR_BASE   -  94 )
#define  CCPErrBadHandle                        ( mc_ERROR_BASE   -  95 )
#define  CCPErrCommandTimeout                   ( mc_ERROR_BASE   -  96 )
#define  CCPErrBadPropertyValue                 ( mc_ERROR_BASE   -  97 )
#define  CCPErrBadParam                         ( mc_ERROR_BASE   -  98 )
#define  CCPErrNoDriver                         ( mc_ERROR_BASE   -  99 )
#define  CCPErrBadSizeOrLength                  ( mc_ERROR_BASE   - 100 )
#define  CCPErrConnectionOpen                   ( mc_ERROR_BASE   - 101 )

#define  XCPErrProgrammingActive                ( mc_ERROR_BASE   - 102 )
#define  XCPErrWriteProtected                   ( mc_ERROR_BASE   - 103 )
#define  XCPErrPageNotValid                     ( mc_ERROR_BASE   - 104 )
#define  XCPErrModeNotValid                     ( mc_ERROR_BASE   - 105 )
#define  XCPErrSegmentNotValid                  ( mc_ERROR_BASE   - 106 )
#define  XCPErrSequence                         ( mc_ERROR_BASE   - 107 )
#define  XCPErrDAQConfig                        ( mc_ERROR_BASE   - 108 )
#define  XCPErrMemoryOverflow                   ( mc_ERROR_BASE   - 109 )
// Errors mc_ERROR_BASE + 110   ... mc_ERROR_BASE + 129  A2L file reader errors
#define  DcsErrorCallbackAlreadyActive          ( mc_ERROR_BASE   - 130 )
#define  DcsErrorFunctionNotAvailable           ( mc_ERROR_BASE   - 131 )
#define  mcErrUnsupportedCmd                    ( mc_ERROR_BASE   - 132 )
#define  mcErrorInvalidName                     ( mc_ERROR_BASE   - 133 )
#define  mcErrorInvalidMappingParameter         ( mc_ERROR_BASE   - 134 )
#define  mcErrorScalingStringTruncated          ( mc_ERROR_BASE   - 135 )
#define  mcErrorScalingInvalidExpression        ( mc_ERROR_BASE   - 136 )
#define  mcErrorScalingInverseNotDefined        ( mc_ERROR_BASE   - 137 )
#define  mcErrorScalingOverlappingIntervals     ( mc_ERROR_BASE   - 138 )
#define  mcErrorScalingIntValNotDefined         ( mc_ERROR_BASE   - 139 )
#define  mcErrorScalingDenominatorZero          ( mc_ERROR_BASE   - 140 )
#define  mcErrorDllLoad                         ( mc_ERROR_BASE   - 141 )
#define  mcErrorInvalidArbitrationID            ( mc_ERROR_BASE   - 142 )
#define  mcErrorPortAlreadyUsed                 ( mc_ERROR_BASE   - 143 )
#define  mcErrorIncompleteFPGAInterface         ( mc_ERROR_BASE   - 144 )
#define  mcErrorBitfileNotFound                 ( mc_ERROR_BASE   - 145 )
#define  mcErrorUnsupportedScalingProperty      ( mc_ERROR_BASE   - 146 )
#define  mcErrorUmbiguousPageMapping            ( mc_ERROR_BASE   - 147 )
#define  mcError_CanDriverError                 ( mc_ERROR_BASE   - 148 )
#define  mcError_BCI_Internal                   ( mc_ERROR_BASE   - 149 )
#define  mcError_DeviceError                    ( mc_ERROR_BASE   - 150 )
#define  mcError_ReadBufferOverflow             ( mc_ERROR_BASE   - 151 )
#define  mcError_WriteBufferOverflow            ( mc_ERROR_BASE   - 152 )
#define  mcError_InvalidFrameFormat             ( mc_ERROR_BASE   - 153 )
#define  mcError_CanFdNotSupported              ( mc_ERROR_BASE   - 154 )
#define  mcError_NiCanNotFound                  ( mc_ERROR_BASE   - 155 )
#define  mcErrorTimingSourceUnavailable         ( mc_ERROR_BASE   - 156 )
#define  mcError_RawMeasNotSupported            ( mc_ERROR_BASE   - 157 )
#define  mcErrorDAQListAlreadySetup             ( mc_ERROR_BASE   - 158 )
#define  mcErrorECUIsConnected                  ( mc_ERROR_BASE   - 159 )
#define  mcErrorXnetIpStackNotSupported         ( mc_ERROR_BASE   - 160 )


// Property ID definitions
// Property Flags
#define  mcMaskPropertyType                                          0xFF00   ///< internal mask to extract the property type from a property ID
#define  mcGenProperty                                               0x0000   ///< general mc property
#define  mcDBProperty                                                0x0100   ///< database related property
#define  mcECUProperty                                               0x0200   ///< ECU related property
#define  mcMeasProperty                                              0x0300   ///< measurement related property
#define  mcCharProperty                                              0x0400   ///< characteristic related property
#define  mcDAQProperty                                               0x0500   ///< DAQ list related property
#define  mcCommProperty                                              0x0600   ///< XCP communication property
#define  mcPGMProperty                                               0x0700   ///< XCP programming related property
#define  mcCRCProperty                                               0x0800   ///< XCP CRC checksum related property
#define  mcOptCmdProperty                                            0x0900   ///< Property for optional CCP command support
#define  mcECUObjProperty                                            0x0A00   ///< Property for GROUP, FUNCTION and more ECU children

// properties for XCP communication
#define  mcIPProperty                           ( mcCommProperty  |  0x0000 ) ///< XCP communication over Ethernet
#define  mcCANProperty                          ( mcCommProperty  |  0x0010 ) ///< XCP communication over CAN
#define  mcGenCommProperty                      ( mcCommProperty  |  0x0040 ) ///< Generic communication property


// Generic:Version (Major)
#define mcPropGen_Version_Major                 ( mcGenProperty   |  0x0000 ) // --r  -- u32 RO Version major number
// Generic:Version (Minor)
#define mcPropGen_Version_Minor                 ( mcGenProperty   |  0x0001 ) // --r  -- u32 RO Version minor number
// Generic:Version (Update)
#define mcPropGen_Version_Update                ( mcGenProperty   |  0x0002 ) // --r  -- u32 RO Version update number
// Generic:Version (Build)
#define mcPropGen_Version_Build                 ( mcGenProperty   |  0x0003 ) // --r  -- u32 RO Version build number
// Generic:Version (String)
#define mcPropGen_Version_Comment               ( mcGenProperty   |  0x0004 ) // --r  -- *char RO Version comment
// Generic:Version (String) Size
#define mcPropGen_Version_Comment_Size          ( mcGenProperty   |  0x0005 ) // --r  -- u32 RO Size of the version comment string
// Database:Filename
#define mcPropDB_Filename                       ( mcDBProperty    |  0x0000 ) // --r  -- *char RO name of the database file
// Database:Filename Size
#define mcPropDB_Filename_Size                  ( mcDBProperty    |  0x0001 ) // --r  -- u32 RO size of name of the database file
// ECU:DTO ID
#define mcPropECU_DTO_ID                        ( mcECUProperty   |  0x0000 ) // --rw -- u32 R/W CAN ID for communication
// ECU:CRO ID
#define mcPropECU_CRO_ID                        ( mcECUProperty   |  0x0001 ) // --rw -- u32 R/W CAN ID for communication
// ECU:Interface
#define mcPropECU_Interface                     ( mcECUProperty   |  0x0003 ) // --r  -- u32 RO CAN interface used for communication
// ECU:Baudrate
#define mcPropECU_BaudRate                      ( mcECUProperty   |  0x0004 ) // --rw -- u32 R/W communication baud rate
// ECU:StationAddress
#define mcPropECU_StationAddress                ( mcECUProperty   |  0x0005 ) // --rw -- u32 R/W station address of the ECU
// ECU:ByteOrder
#define mcPropECU_ByteOrder                     ( mcECUProperty   |  0x0006 ) // --rw -- u32 R/W byte ordering for communication
// ECU:SeedKey Cal
#define mcPropECU_SeedKey_Cal                   ( mcECUProperty   |  0x0007 ) // --rw -- *char R/W name of the seed-key dll to calculate the key for calibration access
// ECU:SeedKey DAQ
#define mcPropECU_SeedKey_DAQ                   ( mcECUProperty   |  0x0008 ) // --rw -- *char R/W name of the seed-key dll to calculate the key for DAQ list access
// ECU:SeedKey Prog
#define mcPropECU_SeedKey_Prog                  ( mcECUProperty   |  0x0009 ) // --rw -- *char R/W name of the seed-key dll to calculate the key for programming
// ECU:SeedKey Checksum
#define mcPropECU_Checksum                      ( mcECUProperty   |  0x000A ) // --rw -- *char R/W name of the dll to calculate checksums
// ECU:Name
#define mcPropECU_Name                          ( mcECUProperty   |  0x000B ) // --r  -- *char RO name of the ECU
// ECU:MasterID
#define mcPropECU_MasterID                      ( mcECUProperty   |  0x000C ) // --rw -- u8[ 6 ] R/W identifier of the master
// ECU:DataType
#define mcPropECU_ID_DataType                   ( mcECUProperty   |  0x000D ) // --r  -- u8 RO data type of the ECU ID (ECU specific)
// ECU:ID Length
#define mcPropECU_ID_Length                     ( mcECUProperty   |  0x000E ) // --r  -- u8 RO length of the ECU ID in byte
// ECU:ID
#define mcPropECU_ID                            ( mcECUProperty   |  0x000F ) // --r  -- u8[] RO ECU ID (ECU specific)
// ECU:SeedKey Cal Size
#define mcPropECU_SeedKey_Cal_Size              ( mcECUProperty   |  0x0010 ) // --r  -- u32 RO size of the name of the seed-key dll to calculate the key for calibration access
// ECU:SeedKey DAQ Size
#define mcPropECU_SeedKey_DAQ_Size              ( mcECUProperty   |  0x0011 ) // --r  -- u32 RO size of the name of the seed-key dll to calculate the key for DAQ list access
// ECU:SeedKey Prog Size
#define mcPropECU_SeedKey_Prog_Size             ( mcECUProperty   |  0x0012 ) // --r  -- u32 RO size of the name of the seed-key dll to calculate the key for programming
// ECU:SeedKey Checksum Size
#define mcPropECU_Checksum_Size                 ( mcECUProperty   |  0x0013 ) // --r  -- u32 RO size of the name of the dll to calculate checksums
// ECU:Name Size
#define mcPropECU_Name_Size                     ( mcECUProperty   |  0x0014 ) // --r  -- u32 RO size of the name of the ECU
// ECU:DTO Task
#define mcPropECU_DTO_Task                      ( mcECUProperty   |  0x0015 ) // --r  -- nctTypeTaskref RO Task reference to the CAN Task assigned to the DTO ID
// ECU:CRO Task
#define mcPropECU_CRO_Task                      ( mcECUProperty   |  0x0016 ) // --r  -- nctTypeTaskref RO Task reference to the CAN Task assigned to the CRO ID
// ECU:Single Byte DAQ Lists
#define mcPropECU_Single_Byte_DAQ_Lists         ( mcECUProperty   |  0x0017 ) // --rw -- u8 R/W flag to determine if an ECU supports single byte or multi byte DAQ list entries
// ECU:TimingFactor
#define mcPropECU_TimingFactor                  ( mcECUProperty   |  0x0018 ) // --rw -- u32 R/W factor for timing adjustments.
// ECU:FirstDAQListToSearch
#define mcPropECU_FirstDAQListToSearch          ( mcECUProperty   |  0x0019 ) // --rw -- u8 R/W Number of the firs DAQ list to search for free DAQ lists.
// ECU:CmdByteOrder
#define mcPropECU_CmdByteOrder                  ( mcECUProperty   |  0x0020 ) // --rw -- u32 R/W byte ordering for multi byte command parameters
// ECU:SeedKey XCP
#define mcPropECU_SeedKey_XCP                   ( mcECUProperty   |  0x0021 ) // --rw -- *char R/W name of the seed-key dll for XCP
// ECU:SeedKey XCP Size
#define mcPropECU_SeedKey_XCP_Size              ( mcECUProperty   |  0x0022 ) // --r  -- *char RO size of the name of the seed-key dll for XCP
// ECU:NumberOfDefinedDAQLists
#define mcPropECU_NumberOfDefinedDAQLists       ( mcECUProperty   |  0x0023 ) // --r  -- u32 RO number of DAQ lists defined in the A2L file (number of SOURCE structs)
// ECU:DAQListNumbers
#define mcPropECU_DAQListNumbers                ( mcECUProperty   |  0x0024 ) // --r  -- u16[] RO the daq list numbers for all DAQ lists defined in the A2L file (each SOURCE's DAQ list number)
// ECU:MappingSource
#define mcPropECU_MappingSource                 ( mcECUProperty   |  0x0025 ) // --rw -- u16 R/W source page for address mapping based on defined pages
// ECU:MappingSourceName
#define mcPropECU_MappingSourceName             ( mcECUProperty   |  0x0026 ) // --rw -- *char R/W name of the source page for address mapping based on defined pages
// ECU:MappingSourceNameLength
#define mcPropECU_MappingSourceNameLength       ( mcECUProperty   |  0x0027 ) // --r  -- u32 RO size of the name of the source page for address mapping based on defined pages
// ECU:MappingTarget
#define mcPropECU_MappingTarget                 ( mcECUProperty   |  0x0028 ) // --rw -- u16 R/W target page for address mapping based on defined pages
// ECU:MappingTargetName
#define mcPropECU_MappingTargetName             ( mcECUProperty   |  0x0029 ) // --rw -- *char R/W name target page for address mapping based on defined pages
// ECU:MappingTargetNameLength
#define mcPropECU_MappingTargetNameLength       ( mcECUProperty   |  0x002A ) // --r  -- u32 RO size of the name of the target page for address mapping based on defined pages
// ECU:MappingMode
#define mcPropECU_MappingMode                   ( mcECUProperty   |  0x002B ) // --rw -- u16 R/W Mode for address mapping - 0: off, - 1: defined pages mapping, - 2: ADDR_MAPPING based address mapping
// ECU:SimDAQList
#define mcPropECU_SimDAQList                    ( mcECUProperty   |  0x002C ) // --rw -- u32 R/W use a simulated DAQ list (polling) instead of a DAQ list on the ECU
// ECU:CCPConnectRetries
#define mcPropECU_CCPConnectRetries             ( mcECUProperty   |  0x002D ) // --rw -- u32 R/W maximum number of retries to connect to a CCP ECU
// ECU:SeedChkDllPath 
#define mcPropECU_SeedChkDllPath                ( mcECUProperty   |  0x002E ) // --rw -- *char R/W path to seedkey and checksum DLL, path of A2L file is used when empty (default)
// ECU:SeedChkDllPath Size 
#define mcPropECU_SeedChkDllPath_Size           ( mcECUProperty   |  0x002F ) // --r  -- u32 RO size of the mcPropECU_SeedChkDllPath string in bytes
// ECU:BufferedSP
#define mcPropECU_BufferedSP                    ( mcECUProperty   |  0x0030 ) // --r  -- u32 RO buffered singlepoint Read is available
// ECU:XCP DAQ LIST Count
#define mcPropECU_XCP_DAQ_LIST_Count            ( mcECUProperty   |  0x0031 ) // --r  -- u32 RO number of DAQ_LISTs within the DAQ struct of the ECU (MODULE)
// ECU:XCP DAQ LIST Numbers
#define mcPropECU_XCP_DAQ_LIST_Numbers          ( mcECUProperty   |  0x0032 ) // --r  -- u32[] RO array with the DAQ_LIST numbers (use mcPropECU_XCPDAQList_Count to find out the number of DAQ_LIST structures)
// ECU:XCP DAQ LIST IsPredefined
#define mcPropECU_XCP_DAQ_LIST_IsPredefined     ( mcECUProperty   |  0x0033 ) // --r  -- u32 RO flag: is the given DAQ_LIST predefined?
// ECU:XCP DAQ LIST MeasNamesSize
#define mcPropECU_XCP_DAQ_LIST_MeasNamesSize    ( mcECUProperty   |  0x0034 ) // --r  -- u32 RO the size which is needed to store MEASUREMENT names assigned to the predefined DAQ_LIST as a list of comma separated values
// ECU:XCP DAQ LIST MeasNames
#define mcPropECU_XCP_DAQ_LIST_MeasNames        ( mcECUProperty   |  0x0035 ) // --r  -- *char RO the MEASUREMENT names assigned to the predefined DAQ_LIST as a list of comma separated values
// ECU:EventChannel
#define mcPropECU_EventChannel                  ( mcECUProperty   |  0x0036 ) // --r  -- i32 RO the event id for the specified event name
// ECU:DAQListMode
#define mcPropECU_DAQListMode                   ( mcECUProperty   |  0x0037 ) // --r  -- u32 RO DAQ list mode: 0=STATIC, 1=DYNAMIC (for XCP, CCP always returns STATIC)
// ECU:LogFileName
#define mcPropECU_LogFileName                   ( mcECUProperty   |  0x0038 ) // --rw -- *char R/W LogFileName: filename to log protocol data traffic to ("": do not log/stop logging, default)
// ECU:LogFileName Size
#define mcPropECU_LogFileName_Size              ( mcECUProperty   |  0x0039 ) // --r  -- u32 RO size of the mcPropECU_LogFileName string in bytes
// ECU:XCP_UseQueryFunctions
#define mcPropECU_XCP_UseQueryFunctions         ( mcECUProperty   |  0x003A ) // --rw -- u32 R/W use ECU query functions to retrieve properties, 0: use A2L file, 1(default): use XCP query functions
// ECU:XCP Timeout T1
#define mcPropECU_XCP_Timeout_T1                ( mcECUProperty   |  0x0041 ) // --rw -- u32 R/W XCP Timeout T1[ms]
// ECU:XCP Timeout T2
#define mcPropECU_XCP_Timeout_T2                ( mcECUProperty   |  0x0042 ) // --rw -- u32 R/W XCP Timeout T2[ms]
// ECU:XCP Timeout T3
#define mcPropECU_XCP_Timeout_T3                ( mcECUProperty   |  0x0043 ) // --rw -- u32 R/W XCP Timeout T3[ms]
// ECU:XCP Timeout T4
#define mcPropECU_XCP_Timeout_T4                ( mcECUProperty   |  0x0044 ) // --rw -- u32 R/W XCP Timeout T4[ms]
// ECU:XCP Timeout T5
#define mcPropECU_XCP_Timeout_T5                ( mcECUProperty   |  0x0045 ) // --rw -- u32 R/W XCP Timeout T5[ms]
// ECU:XCP Timeout T6
#define mcPropECU_XCP_Timeout_T6                ( mcECUProperty   |  0x0046 ) // --rw -- u32 R/W XCP Timeout T6[ms]
// ECU:XCP Timeout T7
#define mcPropECU_XCP_Timeout_T7                ( mcECUProperty   |  0x0047 ) // --rw -- u32 R/W XCP Timeout T7[ms]
// ECU:CCP Timeout Std
#define mcPropECU_CCP_Timeout_Std               ( mcECUProperty   |  0x0048 ) // --rw -- u32 R/W CCP Timeout Std[ms]     (standard commands)
// ECU:CCP Timeout Pgm
#define mcPropECU_CCP_Timeout_Pgm               ( mcECUProperty   |  0x0049 ) // --rw -- u32 R/W CCP Timeout Pgm[ms]     (PROGRAM, PROGRAM_6)
// ECU:CCP Timeout Mem
#define mcPropECU_CCP_Timeout_Mem               ( mcECUProperty   |  0x004A ) // --rw -- u32 R/W CCP Timeout Mem[ms]     (BUILD_CHKSUM, MOVE, CLEAR_MEMORY)
// ECU:CCP Timeout Diag
#define mcPropECU_CCP_Timeout_Diag              ( mcECUProperty   |  0x004B ) // --rw -- u32 R/W CCP Timeout Diag[ms]    (DIAG_SERVICE)
// ECU:CCP Timeout Act
#define mcPropECU_CCP_Timeout_Act               ( mcECUProperty   |  0x004C ) // --rw -- u32 R/W CCP Timeout Act[ms]     (ACTION_SERVICE)
// ECU:Comment
#define mcPropECU_Comment                       ( mcECUProperty   |  0x004D ) // --r  -- *char RO ECU comment
// ECU:Comment Size
#define mcPropECU_Comment_Size                  ( mcECUProperty   |  0x004E ) // --r  -- u32 RO Size of the ECU comment
// ECU:XCP NumSegments
#define mcPropECU_XCP_NumSegments               ( mcECUProperty   |  0x004F ) // --r  -- u8 RO XCP Number of Segments
// ECU:XCP NumPages
#define mcPropECU_XCP_NumPages                  ( mcECUProperty   |  0x0050 ) // --r  -- u8 RO XCP Number of Pages per Segment
// ECU:XCP Address Extension
#define mcPropECU_XCP_AddressExtension          ( mcECUProperty   |  0x0051 ) // --r  -- u8 RO XCP Address Extension per Segment
// ECU:XCP CompressionMethod 
#define mcPropECU_XCP_CompressionMethod         ( mcECUProperty   |  0x0052 ) // --r  -- u8 RO XCP Compression Method per Segment
// ECU:XCP EncryptionMethod
#define mcPropECU_XCP_EncryptionMethod          ( mcECUProperty   |  0x0053 ) // --r  -- u8 RO XCP Encryption Method per Segment
// ECU:XCP PageNo
#define mcPropECU_XCP_PageNo                    ( mcECUProperty   |  0x0054 ) // --r  -- u8 RO XCP Page Number
// ECU:XCP PageECUAccess
#define mcPropECU_XCP_PageECUAccess             ( mcECUProperty   |  0x0055 ) // --r  -- u8 RO XCP Page ECU Access
// ECU:XCP PageXCPReadAccess
#define mcPropECU_XCP_PageXCPReadAccess         ( mcECUProperty   |  0x0056 ) // --r  -- u8 RO XCP Page XCP Read Access
// ECU:XCP PageXCPWriteAccess
#define mcPropECU_XCP_PageXCPWriteAccess        ( mcECUProperty   |  0x0057 ) // --r  -- u8 RO XCP Page XCP Write Access
// ECU:XCP PageInitSegment
#define mcPropECU_XCP_PageInitSegment           ( mcECUProperty   |  0x0058 ) // --r  -- u8 RO XCP Segment that initialized this Page
// ECU:CCP NumPages
#define mcPropECU_CCP_NumPages                  ( mcECUProperty   |  0x0059 ) // --r  -- u32 RO CCP Number of Defined Pages
// ECU:CCP PageNo
#define mcPropECU_CCP_PageNo                    ( mcECUProperty   |  0x005A ) // --r  -- u32 RO CCP Page Number
// ECU:CCP PageFlags
#define mcPropECU_CCP_PageFlags                 ( mcECUProperty   |  0x005B ) // --r  -- u32 RO CCP Page Flags
// ECU:CCP PageAddress
#define mcPropECU_CCP_PageAddress               ( mcECUProperty   |  0x005C ) // --r  -- u32 RO CCP Page Address
// ECU:CCP PageAddressExtension
#define mcPropECU_CCP_PageAddressExtension      ( mcECUProperty   |  0x005D ) // --r  -- u8 RO CCP Page Address Extension
// ECU:MeasSplitAllowed
#define mcPropECU_MeasSplitAllowed              ( mcECUProperty   |  0x005E ) // --rw -- u32 (u32 (boolean)) R/W allow splitting of measurements accross ODTs
// ECU:TransportLayerInstance
#define mcPropECU_TransportLayerInstance        ( mcECUProperty   |  0x005F ) // --rw -- *char R/W to select TRANSPORT_LAYER_INSTANCE in XCPplus nodes
// ECU:TransportLayerInstance Size
#define mcPropECU_TransportLayerInstance_Size   ( mcECUProperty   |  0x0060 ) // --r  -- u32 RO Size of Transport Layer Instance name
// Measurement:Address
#define mcPropMeas_Address                      ( mcMeasProperty  |  0x0000 ) // --r  -- u16 RO Address of the measurement
// Measurement:Extension
#define mcPropMeas_Extension                    ( mcMeasProperty  |  0x0001 ) // --r  -- u8  RO Address extension of the measurement
// Measurement:Datatype
#define mcPropMeas_Datatype                     ( mcMeasProperty  |  0x0002 ) // --r  -- u8  RO Datatype of the measurement
// Measurement:ByteOrder
#define mcPropMeas_ByteOrder                    ( mcMeasProperty  |  0x0003 ) // --rw -- u32 R/W Byte ordering of the measurement
// Measurement:Maximum
#define mcPropMeas_Maximum                      ( mcMeasProperty  |  0x0004 ) // --r  -- f64 RO Maximum value of the measurement
// Measurement:Minimum
#define mcPropMeas_Minimum                      ( mcMeasProperty  |  0x0005 ) // --r  -- f64 RO Minimum value of the measurement
// Measurement:ReadOnly
#define mcPropMeas_ReadOnly                     ( mcMeasProperty  |  0x0006 ) // --r  -- u32 RO Read only flag fro the measurement
// Measurement:IsVirtual
#define mcPropMeas_IsVirtual                    ( mcMeasProperty  |  0x0007 ) // --r  -- u32 RO Measurement is virtual
// Measurement:Unit Size
#define mcPropMeas_Unit_Size                    ( mcMeasProperty  |  0x0008 ) // --r  -- u32 RO size of Unit
// Measurement:Unit
#define mcPropMeas_Unit                         ( mcMeasProperty  |  0x0009 ) // --r  -- *char RO Unit
// Measurement:Comment Size
#define mcPropMeas_Comment_Size                 ( mcMeasProperty  |  0x000A ) // --r  -- u32 RO size of comment string
// Measurement:Comment
#define mcPropMeas_Comment                      ( mcMeasProperty  |  0x000B ) // --r  -- *char RO Comment string
// Measurement:Scale Factor
#define mcPropMeas_Scale_Factor                 ( mcMeasProperty  |  0x000C ) // --r  -- f64 RO scaling factor for linear scaling
// Measurement:Scale Offset
#define mcPropMeas_Scale_Offset                 ( mcMeasProperty  |  0x000D ) // --r  -- f64 RO scaling offset for linear scaling
// Measurement:Scale Type
#define mcPropMeas_Scale_Type                   ( mcMeasProperty  |  0x000E ) // --r  -- u32 RO scaling type of the measurement
// Measurement:Scale TextValues Size
#define mcPropMeas_Scale_TextValues_Size        ( mcMeasProperty  |  0x000F ) // --r  -- u32 RO size of the scaling text values
// Measurement:Scale TextValues
#define mcPropMeas_Scale_TextValues             ( mcMeasProperty  |  0x0010 ) // --r  -- u32 RO scaling text values
// Characteristic:Address
#define mcPropChar_Address                      ( mcCharProperty  |  0x0000 ) // --r  -- u16 RO Address of the characteristic
// Characteristic:Extension
#define mcPropChar_Extension                    ( mcCharProperty  |  0x0001 ) // --r  -- u8  RO Address extension of the characteristic
// Characteristic:Datatype
#define mcPropChar_Datatype                     ( mcCharProperty  |  0x0002 ) // --r  -- u8 RO Datatype of the characteristic
// Characteristic:ByteOrder
#define mcPropChar_ByteOrder                    ( mcCharProperty  |  0x0003 ) // --rw -- u32 R/W Byte ordering of the characteristic
// Characteristic:Dimension
#define mcPropChar_Dimension                    ( mcCharProperty  |  0x0004 ) // --r  -- u32 RO Dimension of the characteristic = d: 0, 1, or 2
// Characteristic:Sizes
#define mcPropChar_Sizes                        ( mcCharProperty  |  0x0005 ) // --r  -- u32[ d ] RO Array sizes for X and Y direction of the characteristic
// Characteristic:X Axis
#define mcPropChar_X_Axis                       ( mcCharProperty  |  0x0006 ) // --r  -- f64[ Sizes[0] ] RO X-Axis values on which the characteristic is defined, valid if d = 1, 2
// Characteristic:Y Axis
#define mcPropChar_Y_Axis                       ( mcCharProperty  |  0x0007 ) // --r  -- f64[ Sizes[1] ] RO Y-Axis values on which the characteristic is defined, valid if d = 2
// Characteristic:Maximum
#define mcPropChar_Maximum                      ( mcCharProperty  |  0x0008 ) // --r  -- f64 RO Maximum value of the characteristic
// Characteristic:Minimum
#define mcPropChar_Minimum                      ( mcCharProperty  |  0x0009 ) // --r  -- f64 RO Minimum value of the characteristic
// Characteristic:ReadOnly
#define mcPropChar_ReadOnly                     ( mcCharProperty  |  0x000A ) // --r  -- u32 RO Read only flag for the characteristic
// Characteristic:Unit Size
#define mcPropChar_Unit_Size                    ( mcCharProperty  |  0x000B ) // --r  -- u32 RO size of Unit
// Characteristic:Unit
#define mcPropChar_Unit                         ( mcCharProperty  |  0x000C ) // --r  -- *char RO Unit
// Characteristic:Comment Size
#define mcPropChar_Comment_Size                 ( mcCharProperty  |  0x000D ) // --r  -- u32 RO size of comment string
// Characteristic:Comment
#define mcPropChar_Comment                      ( mcCharProperty  |  0x000E ) // --r  -- *char RO comment string
// Characteristic:Scale Factor
#define mcPropChar_Scale_Factor                 ( mcCharProperty  |  0x000F ) // --r  -- f64 RO scaling factor for linear scaling
// Characteristic:Scale Offset
#define mcPropChar_Scale_Offset                 ( mcCharProperty  |  0x0010 ) // --r  -- f64 RO scaling offset for linear scaling
// Characteristic:Scale Type
#define mcPropChar_Scale_Type                   ( mcCharProperty  |  0x0011 ) // --r  -- u32 RO scaling type of the characteristic
// Characteristic:Scale TextValues Size
#define mcPropChar_Scale_TextValues_Size        ( mcCharProperty  |  0x0012 ) // --r  -- u32 RO size of the scaling text values
// Characteristic:Scale TextValues
#define mcPropChar_Scale_TextValues             ( mcCharProperty  |  0x0013 ) // --r  -- u32 RO scaling text values
// Group:IsRoot
#define mcPropGroup_IsRoot                      ( mcECUObjProperty |  0x0000 ) // --r  -- u32 RO IsRoot flag
// Group:Comment Size
#define mcPropGroup_Comment_Size                ( mcECUObjProperty |  0x0001 ) // --r  -- u32 RO size of comment string
// Group:Comment
#define mcPropGroup_Comment                     ( mcECUObjProperty |  0x0002 ) // --r  -- *char RO comment string
// Function:Comment Size
#define mcPropFunction_Comment_Size             ( mcECUObjProperty |  0x0003 ) // --r  -- u32 RO size of comment string
// Function:Comment
#define mcPropFunction_Comment                  ( mcECUObjProperty |  0x0004 ) // --r  -- *char RO comment string
// DAQ:Mode
#define mcPropDAQ_Mode                          ( mcDAQProperty   |  0x0000 ) // --rw -- u32 R/W data acquisition mode for the DAQ task. May be mcDAQModeDAQList or mcDAQModePolling
// DAQ:DTO ID
#define mcPropDAQ_DTO_ID                        ( mcDAQProperty   |  0x0001 ) // --rw -- u32 R/W DTO ID used for transmitting the DAQ list data
// DAQ:SampleRate
#define mcPropDAQ_SampleRate                    ( mcDAQProperty   |  0x0002 ) // --r  -- f64 RO Sampling rate to read the DAQ list data from the bus
// DAQ:NumChannels
#define mcPropDAQ_NumChannels                   ( mcDAQProperty   |  0x0003 ) // --r  -- u32 RO number of "scaled" channels (f64) defined for the DAQ task.
// DAQ:EventChannelName
#define mcPropDAQ_EventChannelName              ( mcDAQProperty   |  0x0004 ) // --rw -- string  R/W Name of the event channel assigned to the DAQ Task. As an alternative you can enter the number of the event channel as an ASCII String ( "0" <= eventchannel <= "255").
// DAQ:DTO Task
#define mcPropDAQ_DTO_Task                      ( mcDAQProperty   |  0x0005 ) // --r  -- nctTypeTaskref RO Task reference to the CAN Task assigned to the DTO ID of the DAQ task
// DAQ:Prescaler
#define mcPropDAQ_Prescaler                     ( mcDAQProperty   |  0x0006 ) // --rw -- u16 R/W Prescaler for the DAQ task on the ECU
// DAQ:EventChannelName Size
#define mcPropDAQ_EventChannelName_Size         ( mcDAQProperty   |  0x0007 ) // --r  -- u32 RO Size of the Event channel name
// DAQ:SamplesPending
#define mcPropDAQ_SamplesPending                ( mcDAQProperty   |  0x0008 ) // --r  -- u32 RO Number of currently available samples for the DAQ task
// DAQ:MaxLength
#define mcPropDAQList_MaxLength                 ( mcDAQProperty   |  0x0009 ) // --r  -- u16 RO the maximal length of the DAQ list (LENGTH value from the SOURCE struct)
// DAQ:CANIdSelectMode
#define mcPropDAQList_CANIdSelectMode           ( mcDAQProperty   |  0x000A ) // --r  -- u32 RO how to select the CAN Id for the specified DAQ list (0: CAN_ID_FIXED, 1: CAN_ID_VARIABLE, 2: Use DTO ID)
// DAQ:CANId
#define mcPropDAQList_CANId                     ( mcDAQProperty   |  0x000B ) // --r  -- u32 RO the CAN Id for the specified DAQ list (if the SOURCE struct has the CAN_ID_FIXED keyword)
// DAQ:FirstPID
#define mcPropDAQList_FirstPID                  ( mcDAQProperty   |  0x000C ) // --r  -- u8  RO the first Packet Id for the specified DAQ list (if the SOURCE struct has the FIRST_PID keyword)
// DAQ:NumberOfEventChannels
#define mcPropDAQList_NumberOfEventChannels     ( mcDAQProperty   |  0x000D ) // --r  -- u32 RO the number of allowed event channels for the specified DAQ list (RASTER keyword in the SOURCE struct)
// DAQ:EventChannels
#define mcPropDAQList_EventChannels             ( mcDAQProperty   |  0x000E ) // --r  -- u8[] RO the array containing the event channel numbers referenced by the DAQ list (RASTER keyword in the SOURCE struct)
// DAQ:ReductionAllowed
#define mcPropDAQList_ReductionAllowed          ( mcDAQProperty   |  0x000F ) // --r  -- u32 RO the flag if the specified DAQ list allows reduction or not (if the REDUCTION_ALLOWED keyword is present in the SOURCE struct)
// DAQ:NumberOfExcludedDAQLists
#define mcPropDAQList_NumberOfExcludedDAQLists  ( mcDAQProperty   |  0x0010 ) // --r  -- u32 RO the length of the array containing the numbers of DAQ lists not working together with the current DAQ list (EXCLUSIVE keyword in the SOURCE struct)
// DAQ:ExcludedDAQLists
#define mcPropDAQList_ExcludedDAQLists          ( mcDAQProperty   |  0x0011 ) // --r  -- u16[] RO the array containing the numbers of DAQ lists not working together with the current DAQ list (EXCLUSIVE keyword in the SOURCE struct)
// DAQ:ExtSampClk
#define mcPropDAQList_ExtSampClk                ( mcDAQProperty   |  0x0012 ) // --rw -- u32 R/W enable external sample clock for frame API (RTSI 0)
// DAQ:SimDAQListInterval
#define mcPropDAQList_SimDAQListInterval        ( mcDAQProperty   |  0x0014 ) // --rw -- u32 R/W timing interval for polling the simulated DAQ list.
// DAQ:SimExtSampClk
#define mcPropDAQList_SimExtSampClk             ( mcDAQProperty   |  0x0015 ) // --rw -- u32 R/W enable external sample clock for a simulated DAQ list
// DAQ:TimeSinceLastFrame
#define mcPropDAQ_TimeSinceLastFrame            ( mcDAQProperty   |  0x0016 ) // --r  -- f64 RO age of the last DAQ frame received
// DAQ:Name
#define mcPropDAQList_Name                      ( mcDAQProperty   |  0x0017 ) // --r  -- string RO Daq list name
// DAQ:Name Size
#define mcPropDAQList_Name_Size                 ( mcDAQProperty   |  0x0018 ) // --r  -- u32 RO Size of daq list name
// DAQ:DisplayIdentifier
#define mcPropDAQList_DisplayIdentifier         ( mcDAQProperty   |  0x0019 ) // --r  -- string RO Daq list display idenitfier
// DAQ:DisplayIdentifier Size
#define mcPropDAQList_DisplayIdentifier_Size    ( mcDAQProperty   |  0x001A ) // --r  -- u32 RO Size of daq list display identifier
// DAQ:NumDAQPIDs
#define mcPropDAQ_NumDAQPIDs                    ( mcDAQProperty   |  0x001B ) // --r  -- u32 RO Number of DAQ PIDs (frames) configured in this session
// DAQ:TimingSource
#define mcPropDAQ_TimingSource                  ( mcDAQProperty   |  0x001C ) // --rw -- u32 R/W Ring for DAQ Timing Source (0=Automatic, 1=Host Timing, 2=CAN Timing, 3=ECU Timing, ...)
// DAQ:NumRawChannels
#define mcPropDAQ_NumRawChannels                ( mcDAQProperty   |  0x001D ) // --r  -- u32 RO number of raw channels (u64) defined for the DAQ task.
// IP:Address
#define mcPropIPAddress                         ( mcIPProperty    |  0x0000 ) // --rw -- *char R/W IP address for XCP on Ethernet
// IP:Address Size
#define mcPropIPAddress_Size                    ( mcIPProperty    |  0x0001 ) // --r  -- u32 RO Size of the IP address
// IP:Port
#define mcPropIPPort                            ( mcIPProperty    |  0x0002 ) // --rw -- u16 R/W IP port for XCP on Ethernet
// IP:SourcePort
#define mcPropIPSourcePort                      ( mcIPProperty    |  0x0003 ) // --rw -- u32 R/W IP source port for XCP on Ethernet (-1 = any)
// IP:SourceAddress
#define mcPropIPSourceAddress                   ( mcIPProperty    |  0x0004 ) // --rw -- *char R/W IP source address for XCP on Ethernet
// IP:SourceAddress Size
#define mcPropIPSourceAddress_Size              ( mcIPProperty    |  0x0005 ) // --r  -- u32 RO Size of the IP source address
// IP:XnetStackName
#define mcPropIPXnetStackName                   ( mcIPProperty    |  0x0006 ) // --rw -- *char R/W IP NI-XNET Stack Name for IP stack
// IP:XnetStackName Size
#define mcPropIPXnetStackName_Size              ( mcIPProperty    |  0x0007 ) // --r  -- u32 RO Size of the Xnet Stack Name
// IP:XnetLocalInterface
#define mcPropIPXnetLocalInterface              ( mcIPProperty    |  0x0008 ) // --rw -- *char R/W IP NI-XNET Local Interface for IP stack
// IP:XnetLocalInterface Size
#define mcPropIPXnetLocalInterface_Size         ( mcIPProperty    |  0x0009 ) // --r  -- u32 RO Size of the IP local interface
// XCP on CAN:DTOID
#define mcPropDTOID                             ( mcCANProperty   |  0x0000 ) // --rw -- u32 R/W DTO ID for XCP on CAN
// XCP on CAN:CROID
#define mcPropCROID                             ( mcCANProperty   |  0x0001 ) // --rw -- u32 R/W CRO ID for XCP on CAN
// XCP on CAN:CANBaudrate
#define mcPropCANBaudrate                       ( mcCANProperty   |  0x0002 ) // --rw -- u32 (legacy, use mcPropCANBaudrate64) Baudrate for XCP on CAN
// XCP on CAN:CommunicationAPI
#define mcPropCAN_CommunicationAPI              ( mcCANProperty   |  0x0003 ) // --r  -- u8 RO type of NI-CAN API used for communication. See mcTypeCommApi
// XCP on CAN:FrameObjHandle
#define mcPropCAN_FrameObjHandle                ( mcCANProperty   |  0x0004 ) // --r  -- u32 RO CAN ObjHandle from NI-CAN frame API
// XCP on CAN:Termination
#define mcPropCANTermination                    ( mcCANProperty   |  0x0005 ) // --rw -- u32 R/W CAN Termination (for XNET only, 0: not terminated, 1: terminated, anything else: ignored)
// XCP on CAN:Baudrate64
#define mcPropCANBaudrate64                     ( mcCANProperty   |  0x0006 ) // --rw -- u64 R/W Baudrate for XCP on CAN
// XCP on CAN:FDBaudrate64
#define mcPropCANFDBaudrate64                   ( mcCANProperty   |  0x0007 ) // --rw -- u64 R/W CAN FD Baudrate for XCP
// XCP on CAN:IOmode
#define mcPropCANIOmode                         ( mcCANProperty   |  0x0008 ) // --rw -- u32 (enum TmcCanIoMode) R/W CAN I/O mode for XCP on CAN
// XCP on CAN:UseMaxDLC
#define mcPropCANUseMaxDLC                      ( mcCANProperty   |  0x0009 ) // --rw -- u32 (boolean) R/W Use MAX DLC with CAN
// XCP on CAN:FDMaxDLC
#define mcPropCANFDMaxDLC                       ( mcCANProperty   |  0x000A ) // --rw -- u32 R/W CAN FD MAX DLC value
// XCP on CAN:FDUseMaxDLC
#define mcPropCANFDUseMaxDLC                    ( mcCANProperty   |  0x000B ) // --rw -- u32 (boolean) R/W Use CAN FD MAX DLC with CAN FD
// XCP on CAN:ActCANIOmode
#define mcPropActCANIOmode                      ( mcCANProperty   |  0x000C ) // --r  -- u32 RO (TmcCanIoMode) Actual used CAN I/O mode for XCP on CAN
// ECU:LowLatencyMode
#define mcProp_LowLatencyMode                   ( mcGenCommProperty | 0x0000) // --rw -- u32 Enables low latency mode for response commands
// Programming:CompressionMethod
#define mcPropPGM_CompressionMethod             ( mcPGMProperty   |  0x0000 ) // --rw -- u8  R/W compression method for programming. 0=uncompressed
// Programming:EncryptionMethod
#define mcPropPGM_EncryptionMethod              ( mcPGMProperty   |  0x0001 ) // --rw -- u8  R/W encryption  method for programming. 0=uncrypted
// Programming:ProgrammingMethod
#define mcPropPGM_ProgrammingMethod             ( mcPGMProperty   |  0x0002 ) // --rw -- u8  R/W programming method for programming. 0=sequential
// Programming:AccessMethod
#define mcPropPGM_AccessMethod                  ( mcPGMProperty   |  0x0003 ) // --rw -- u8  R/W access      method for programming. 0=absolute, 1=functional
// CRC:BitReflection
#define mcPropCRC_BitReflection                 ( mcCRCProperty   |  0x0000 ) // --rw -- u32 R/W bit reflection for CRC. 0=default 1=reflected
// CRC:StartValue
#define mcPropCRC_StartValue                    ( mcCRCProperty   |  0x0001 ) // --rw -- u32 R/W CRC start value. Default is 0 for CRC_16, 0xFFFF for CRC_16_CCITT, 0xFFFFFFFF for CRC_32.
// Optional Command:GET_SEED
#define  mcPropOptCmd_GET_SEED                  ( mcOptCmdProperty|  0x0000 ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command GET_SEED
// Optional Command:UNLOCK
#define  mcPropOptCmd_UNLOCK                    ( mcOptCmdProperty|  0x0001 ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command UNLOCK
// Optional Command:DNLOAD_6
#define  mcPropOptCmd_DNLOAD_6                  ( mcOptCmdProperty|  0x0002 ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command DNLOAD_6
// Optional Command:SHORT_UP
#define  mcPropOptCmd_SHORT_UP                  ( mcOptCmdProperty|  0x0003 ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command SHORT_UP
// Optional Command:SELECT_CAL_PAGE
#define  mcPropOptCmd_SELECT_CAL_PAGE           ( mcOptCmdProperty|  0x0004 ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command SELECT_CAL_PAGE
// Optional Command:SET_S_STATUS
#define  mcPropOptCmd_SET_S_STATUS              ( mcOptCmdProperty|  0x0005 ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command SET_S_STATUS
// Optional Command:GET_S_STATUS
#define  mcPropOptCmd_GET_S_STATUS              ( mcOptCmdProperty|  0x0006 ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command GET_S_STATUS
// Optional Command:BUILD_CHKSUM
#define  mcPropOptCmd_BUILD_CHKSUM              ( mcOptCmdProperty|  0x0007 ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command BUILD_CHKSUM
// Optional Command:CLEAR_MEMORY
#define  mcPropOptCmd_CLEAR_MEMORY              ( mcOptCmdProperty|  0x0008 ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command CLEAR_MEMORY
// Optional Command:PROGRAM
#define  mcPropOptCmd_PROGRAM                   ( mcOptCmdProperty|  0x0009 ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command PROGRAM
// Optional Command:PROGRAM_6
#define  mcPropOptCmd_PROGRAM_6                 ( mcOptCmdProperty|  0x000A ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command PROGRAM_6
// Optional Command:MOVE
#define  mcPropOptCmd_MOVE                      ( mcOptCmdProperty|  0x000B ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command MOVE
// Optional Command:TEST
#define  mcPropOptCmd_TEST                      ( mcOptCmdProperty|  0x000C ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command TEST
// Optional Command:GET_ACTIVE_CAL_PAGE
#define  mcPropOptCmd_GET_ACTIVE_CAL_PAGE       ( mcOptCmdProperty|  0x000D ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command GET_ACTIVE_CAL_PAGE
// Optional Command:START_STOP_ALL
#define  mcPropOptCmd_START_STOP_ALL            ( mcOptCmdProperty|  0x000E ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command START_STOP_ALL
// Optional Command:DIAG_SERVICE
#define  mcPropOptCmd_DIAG_SERVICE              ( mcOptCmdProperty|  0x000F ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command DIAG_SERVICE
// Optional Command:ACTION_SERVICE
#define  mcPropOptCmd_ACTION_SERVICE            ( mcOptCmdProperty|  0x0010 ) // --rw -- u32 R/W Flag to indicate whether the ECU supports the optional CCP Command ACTION_SERVICE
// Optional Command:EXCHANGE_ID
#define  mcPropCmd_EXCHANGE_ID                  ( mcOptCmdProperty|  0x0011 ) // --rw -- u32 RW Flag to indicate whether the EXCHANGE_ID command should be suppressed during connection to the ECU or not.
// Debug:File
#define mcPropDebugFile                         (                    0xFFFE ) // --w  -- *char WO Name of a debug file

#define  MAXNamesLength                       255           ///< maximum length of ECU, measurement or characteristic names supported
#define  MAXDAQTASKS                          256           ///< maximum number of DAQTasks supported
#define  MAX_DATABASES                         16           ///< maximum number of supported databases

#define  MCINVALIDENUM                       0xFF           ///< Value for an uninitialized enum

// Constants that define the byte ordering
#define  mcByteorder_MSB_Last                   0
#define  mcByteorder_MSB_First                  1

// Constants that define the data types defined in the A2L file
#define  mcDatatype_u8                          0
#define  mcDatatype_s8                          1
#define  mcDatatype_u16                         2
#define  mcDatatype_s16                         3
#define  mcDatatype_u32                         4
#define  mcDatatype_s32                         5
#define  mcDatatype_f32                         6
#define  mcDatatype_f64                         7
#define  mcDatatype_u64                         8
#define  mcDatatype_s64                         9

// Constants defining the scaling types
#define mcScalingType_Unknown                   ((u32)0)
#define mcScalingType_RationalFunction          ((u32)1)
#define mcScalingType_EnumText                  ((u32)2)
#define mcScalingType_RangeText                 ((u32)3)
#define mcScalingType_Formula                   ((u32)4)
#define mcScalingType_IntTable                  ((u32)5)
#define mcScalingType_NoIntTable                ((u32)6)

// Enum for mcGetPropertyDataType
typedef enum
{
   PropertyDataType_u8,
   PropertyDataType_i8,
   PropertyDataType_u16,
   PropertyDataType_i16,
   PropertyDataType_u32,
   PropertyDataType_i32,
   PropertyDataType_f32,
   PropertyDataType_f64,
   PropertyDataType_string,
   PropertyDataType_unknown
} tPropertyDataType;

typedef enum
{
   mcDAQModeDAQList              = 0,
   mcDAQModePolling              = 1,
   mcDAQModeSTIMList             = 2,
   mcDAQModeDAQListTimeStamped   = 3,
   mcDAQModeUninitialized        = 0xFFFFFFFF
}  TmcMeasuremode;

typedef enum
{
   mcA2LNodeTypeUnknown = 0,
   mcA2LNodeTypeLiteral = 1,
   mcA2LNodeTypeIdentifier = 2,
   mcA2LNodeTypeData = 3,
   mcA2LNodeTypeBlock = 4
}  TmcA2LNodeType;

typedef enum
{
  mcStartStopModeStop            = 0,
  mcStartStopModeStart           = 1,
  mcStartStopModeTransmitDAQ     = 2,
  mcStartStopModePrepareStartAll = 3,
  mcStartStopModeStartAll        = 4,
  mcStartStopModeStopAll         = 5
}  TmcStartStopMode;

// mcPropCANIOmode
typedef enum
{
   mcCANioMode_CAN            = 0,
   mcCANioMode_CAN_FD         = 1,
   mcCANioMode_CAN_FD_BRS     = 2,
   mcCANioMode_ECU_Determined = 3
} TmcCanIoMode;

// mcPropDAQ_TimingSource
typedef enum
{
   mcDAQTimingSource_Automatic      = 0,
   mcDAQTimingSource_Host           = 1,
   mcDAQTimingSource_CAN            = 2,
   mcDAQTimingSource_ECU            = 3
} TmcDAQTimingSource;

#ifdef __cplusplus
   extern "C" {
#endif

#ifndef _NI_intptr_t_DEFINED_
   #define _NI_intptr_t_DEFINED_
   #if ( _MSC_VER == 1200 )
      // This is Windows only
      #if defined(_WIN64)
         typedef i64 intptr_t;
      #else
         typedef i32 intptr_t;
      #endif
   #endif
#endif

typedef u32 mcTypeTaskRef;   ///< Handle for all mc tasks
typedef i32 mcTypeStatus;    ///< Error status

typedef intptr_t mcTypeA2LNode;   ///< Handle used to represent A2L nodes

#define INVALID_A2L_NODE ((mcTypeA2LNode)0)

typedef struct
{
   u32   Address;    ///< Address in the ECU memory
   u8    Extension;  ///< Address extenstion
} mcAddress;  ///< defines a memory address inside the ECU

/************************************************************************/
/*  Functions common to CCP and XCP                                     */
/************************************************************************/




MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDatabaseOpenEx(
     cstr             Database,
     mcTypeTaskRef   *DBRefNum,
     cstr             Reserved
  )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDatabaseOpen(
      cstr             Database,
      mcTypeTaskRef   *DBRefNum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDatabaseOpenFromBuffer(
      cstr             DatabaseName,
      cstr             Buffer,
      u32              SizeOfBuffer,
      mcTypeTaskRef   *DBRefNum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDatabaseClose(
      mcTypeTaskRef *DBRefNum
      )
VX_EXPORT_SYMBOL;




MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcECUSelect(
      mcTypeTaskRef   DBRefNum,
      cstr            ECUName,
      i32             Interface,
      mcTypeTaskRef  *ECURefNum
      )
VX_EXPORT_SYMBOL;



MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcECUSelectEx(
      mcTypeTaskRef    DBRefNum,
      cstr             ECUName,
      cstr             Interface,
      mcTypeTaskRef   *ECURefNum
      )
VX_EXPORT_SYMBOL;



MS_EXPORT_SYMBOL
mcTypeStatus CALLING_CONVENTION mcECUSelect2(
   mcTypeTaskRef    DBRefNum,
   cstr             ECUName,
   u32              Protocol,
   cstr             Interface,
   cstr             InterfaceSpecifier,
   mcTypeTaskRef   *ECURefNum
   )
VX_EXPORT_SYMBOL;



MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcECUConnect(
      mcTypeTaskRef ECURefNum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcECUConnect2(
      mcTypeTaskRef ECURefNum,
      u8            ECUMode
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcECUDisconnect(
      mcTypeTaskRef ECURefNum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcECUDeselect(
      mcTypeTaskRef *ECURefNum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcECUSetCalibrationPage(
      mcTypeTaskRef ECURefNum,
      u8            useRAM,
      u8            mapAddresses
      )
VX_EXPORT_SYMBOL;





MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcGetNames(
      mcTypeTaskRef  RefNum,
      u32            Type,
      cstr           ECUName,
      u32            SizeOfNamesList,
      str            NamesList
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcGetNamesLength(
      mcTypeTaskRef  RefNum,
      u32            Type,
      cstr           ECUName,
      u32           *SizeOfNamesList
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcGetProperty(
      mcTypeTaskRef   RefNum,
      cstr            Name,
      u32             PropertyID,
      u32             SizeOfValue,
      void           *Value
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcSetProperty(
      mcTypeTaskRef   RefNum,
      cstr            Name,
      u32             PropertyID,
      u32             SizeOfValue,
      void           *Value
      )
VX_EXPORT_SYMBOL;





MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDAQInitialize(
      cstr             MeasurementNames,
      mcTypeTaskRef    ECURefNum,
      i32              DAQMode,
      u32              DTO_ID,
      f64              SampleRate,
      mcTypeTaskRef   *DAQRefNum
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
mcTypeStatus CALLING_CONVENTION mcDAQInitializeEx(
   cstr             MeasurementNames,
   cstr             RawMeasurementNames,
   mcTypeTaskRef    ECURefNum,
   i32              DAQMode,
   u32              DTO_ID,
   f64              SampleRate,
   mcTypeTaskRef   *DAQRefNum
)
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDAQListInitialize(
      cstr             MeasurementNames,
      mcTypeTaskRef    ECURefNum,
      i16              DAQListNo,
      i32              DAQMode,
      u32              DTO_ID,
      f64              SampleRate,
      mcTypeTaskRef   *DAQRefNum
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
mcTypeStatus CALLING_CONVENTION mcDAQListInitializeEx(
   cstr             MeasurementNames,
   cstr             RawMeasurementNames,
   mcTypeTaskRef    ECURefNum,
   i16              DAQListNo,
   i32              DAQMode,
   u32              DTO_ID,
   f64              SampleRate,
   mcTypeTaskRef   *DAQRefNum
)
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDAQStartStop(
      mcTypeTaskRef     DAQRefNum,
      u32               StartStopMode
      )
VX_EXPORT_SYMBOL;



MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDAQStart(
      mcTypeTaskRef         DAQRefNum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDAQStop(
      mcTypeTaskRef          DAQRefNum
      )
VX_EXPORT_SYMBOL;



MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDAQRead(
      mcTypeTaskRef       DAQRefNum,
      u32                 NumberOfSamplesToRead,
      mcTypeTimestamp    *StartTime,
      mcTypeTimestamp    *DeltaTime,
      f64                *SampleArray,
      u32                *NumberOfSamplesReturned
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
mcTypeStatus CALLING_CONVENTION mcDAQReadEx(
   mcTypeTaskRef       DAQRefNum,
   u32                 NumberOfSamplesToRead,
   mcTypeTimestamp    *StartTime,
   mcTypeTimestamp    *DeltaTime,
   f64                *SampleArray,
   u64                *RawSampleArray,
   u32                *NumberOfSamplesReturned
)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDAQReadTimestamped(
      mcTypeTaskRef  DAQRefNum,
      u32            NumberOfSamplesToRead,
      i64           *TimestampArray,
      double        *SampleArray,
      u32           *NumberOfSamplesReturned
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
mcTypeStatus CALLING_CONVENTION mcDAQReadTimestampedEx(
   mcTypeTaskRef  DAQRefNum,
   u32            NumberOfSamplesToRead,
   i64           *TimestampArray,
   double        *SampleArray,
   u64           *RawSampleArray,
   u32           *NumberOfSamplesReturned
)
VX_EXPORT_SYMBOL;

// XCHG D AQWrite/DAQ Write
// PXCH D AQRef Num/DAQ Ref Num
MS_EXPORT_SYMBOL
  mcTypeStatus CALLING_CONVENTION mcDoubleToText(
    mcTypeTaskRef ECURefNum,
    u32 ObjectType,
    cstr ObjectName,
    double Value,
    u32 SizeOfTextValue,
    str TextValue)
VX_EXPORT_SYMBOL;

// XCHG D AQWrite/DAQ Write
// PXCH D AQRef Num/DAQ Ref Num
MS_EXPORT_SYMBOL
  mcTypeStatus CALLING_CONVENTION mcTextToDouble(
    mcTypeTaskRef ECURefNum,
    u32 ObjectType,
    cstr ObjectName,
    cstr TextValue,
    double *Value)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDAQWrite(
      mcTypeTaskRef       DAQRefNum,
      u32                 NumberOfSamplesToWrite,
      f64                *SampleArray
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
mcTypeStatus CALLING_CONVENTION mcDAQWriteEx(
   mcTypeTaskRef       DAQRefNum,
   u32                 NumberOfSamplesToWrite,
   f64                *SampleArray,
   u64                *RawSampleArray
)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDAQClear(
      mcTypeTaskRef *DAQRefNum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcMeasurementRead(
      mcTypeTaskRef       ECURefNum,
      char               *MeasurementName,
      f64                *Value
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
mcTypeStatus CALLING_CONVENTION mcMeasurementReadRaw(
   mcTypeTaskRef       ECURefNum,
   char               *MeasurementName,
   u64                *Value
)
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcMeasurementWrite(
      mcTypeTaskRef      ECURefNum,
      char              *MeasurementName,
      f64                Value
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
mcTypeStatus CALLING_CONVENTION mcMeasurementWriteRaw(
   mcTypeTaskRef      ECURefNum,
   char              *MeasurementName,
   u64                Value
)
VX_EXPORT_SYMBOL;





MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCharacteristicRead(
      mcTypeTaskRef       ECURefNum,
      char               *CharacteristicName,
      f64                *Values,
      u32                 NumberOfValues
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCharacteristicReadRaw(
      mcTypeTaskRef       ECURefNum,
      char               *CharacteristicName,
      u64                *Values,
      u32                 NumberOfValues
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCharacteristicReadByteArray(
      mcTypeTaskRef       ECURefNum,
      char               *CharacteristicName,
      u8                 *Value,
      u32                 NumberOfValues
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCharacteristicReadSingleValue(
      mcTypeTaskRef        ECURefNum,
      char                *CharacteristicName,
      f64                 *Value,
      u32                  X,
      u32                  Y
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCharacteristicReadRawSingleValue(
      mcTypeTaskRef        ECURefNum,
      char                *CharacteristicName,
      u64                 *Value,
      u32                  X,
      u32                  Y
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCharacteristicWrite(
      mcTypeTaskRef      ECURefNum,
      char              *CharacteristicName,
      f64               *Values,
      u32                NumberOfValues
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCharacteristicWriteRaw(
      mcTypeTaskRef      ECURefNum,
      char              *CharacteristicName,
      u64               *Values,
      u32                NumberOfValues
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCharacteristicWriteByteArray(
      mcTypeTaskRef      ECURefNum,
      char              *CharacteristicName,
      u8                *Values,
      u32                NumberOfValues
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCharacteristicWriteSingleValue(
      mcTypeTaskRef   ECURefNum,
      char           *CharacteristicName,
      f64             Value,
      u32             X,
      u32             Y
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCharacteristicWriteRawSingleValue(
      mcTypeTaskRef   ECURefNum,
      char           *CharacteristicName,
      u64             Value,
      u32             X,
      u32             Y
      )
VX_EXPORT_SYMBOL;





MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcDownload(
      mcTypeTaskRef   ECURefNum,
      mcAddress       Address,
      u32             BlockSize,
      u8             *Data
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcUpload(
      mcTypeTaskRef  ECURefNum,
      mcAddress      Address,
      u32            BlockSize,
      u8            *Data
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcClearMemory(
      mcTypeTaskRef ECURefNum,
      mcAddress     Address,
      u32           BlockSize
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcProgram(
      mcTypeTaskRef  ECURefNum,
      mcAddress      Address,
      u32            BlockSize,
      u8            *Data
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcBuildChecksum(
      mcTypeTaskRef   ECURefNum,
      mcAddress       Address,
      u32             DataSize,
      u8             *ChecksumType,
      u8             *SizeOfChecksum,
      u32            *Checksum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCalculateChecksum(
      mcTypeTaskRef   ECURefNum,
      u32             BlockSize,
      u8             *Data,
      u8              TypeOfChecksum,
      u8             *SizeOfChecksum,
      u32            *Checksum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcProgramStart(
      mcTypeTaskRef   ECURefNum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcProgramReset(
      mcTypeTaskRef   ECURefNum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcGeneric(
      mcTypeTaskRef   ECURefNum,
      u8              Command,
      u8             *Data,
      u32             DataSize,
      u32             Timeout,
      u8             *ErrorCode,
      u8             *ReturnValue,
      u32            *ReturnValueSize
      )
VX_EXPORT_SYMBOL;




/************************************************************************/
/* CCP specific functions                                               */
/************************************************************************/
MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPGetResult(
      mcTypeTaskRef    ECURefNum,
      u32              BlockSize,
      u8              *Data
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPSelectCalPage(
      mcTypeTaskRef ECURefNum,
      mcAddress     Address
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPSetSessionStatus(
      mcTypeTaskRef ECURefNum,
      u8            SessionStatus
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPGetSessionStatus(
      mcTypeTaskRef  ECURefNum,
      u8            *SessionStatus,
      u8            *StatusQualifier,
      u8            *AdditionalStatus
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPMoveMemory(
      mcTypeTaskRef ECURefNum,
      mcAddress     Source,
      mcAddress     Destination,
      u32           BlockSize
      )
VX_EXPORT_SYMBOL;



MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPDiagService(
      mcTypeTaskRef  ECURefNum,
      u16            ServiceNo,
      u8               Params[ 4 ],
      u8            *ResultLength,
      u8            *DataType
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPActionService(
      mcTypeTaskRef  ECURefNum,
      u16            ServiceNo,
      u8             Params[ 4 ],
      u8            *ResultLength,
      u8            *DataType
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPGetVersion(
      mcTypeTaskRef  ECURefNum,
      u8            *MajorVersion,
      u8            *MinorVersion
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPGetActiveCalPage(
      mcTypeTaskRef  ECURefNum,
      mcAddress     *Address
      )
VX_EXPORT_SYMBOL;



MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPDownload(
      mcTypeTaskRef  ECURefNum,
      mcAddress      Address,
      u32            BlockSize,
      u8            *Data
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPUpload(
      mcTypeTaskRef    ECURefNum,
      mcAddress        Address,
      u32              BlockSize,
      u8              *Data
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPCalculateChecksum(
      mcTypeTaskRef   ECURefNum,
      u32             BlockSize,
      u8             *Data,
      u8             *SizeOfChecksum,
      u8             *Checksum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPClearMemory(
      mcTypeTaskRef ECURefNum,
      mcAddress     Address,
      u32           BlockSize
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPProgram(
      mcTypeTaskRef  ECURefNum,
      mcAddress      Address,
      u32            BlockSize,
      u8            *Data
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPBuildChecksum(
      mcTypeTaskRef   ECURefNum,
      mcAddress       Address,
      u32             BlockSize,
      u8             *SizeOfChecksum,
      u8             *Checksum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcCCPGeneric(
      mcTypeTaskRef   ECURefNum,
      u8              Command,
      u8             *Data,
      u32             Timeout,
      u8             *ErrorCode,
      u8             *ReturnValue
      )
VX_EXPORT_SYMBOL;






/************************************************************************/
/* XCP specific functions                                               */
/************************************************************************/
MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcXCPGetStatus(
      mcTypeTaskRef  ECURefNum,
      u8            *SessionStatus,
      u8            *ResourceMask,
      u16           *SessionId
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcXCPGetId(
      mcTypeTaskRef   ECURefNum,
      u8              Type,
      u32            *Length,
      char           *Id
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcXCPSetRequest(
      mcTypeTaskRef   ECURefNum,
      u8              Mode,
      u16             SessionId
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcXCPProgramPrepare(
      mcTypeTaskRef   ECURefNum,
      mcAddress       Address,
      u16             CodeSize
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcXCPProgramVerify(
      mcTypeTaskRef ECURefNum,
      u8            Mode,
      u16           VerType,
      u32           VerValue
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcXCPSetCalPage(
      mcTypeTaskRef   ECURefNum,
      u8              Mode,
      u8              Segment,
      u8              Page
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcXCPGetCalPage(
      mcTypeTaskRef   ECURefNum,
      u8              Mode,
      u8              Segment,
      u8             *Page
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcXCPCopyCalPage(
      mcTypeTaskRef  ECURefNum,
      u8             SourceSegment,
      u8             SourcePage,
      u8             DestinationSegment,
      u8             DestinationPage
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcXCPSetSegmentMode(
      mcTypeTaskRef  ECURefNum,
      u8             Segment,
      u8             Mode
      )
VX_EXPORT_SYMBOL;




MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcMeasurementCreate(
      mcTypeTaskRef   ECURefNum,
      char           *MeasurementName,
      mcAddress       Address,
      i32             DataType,
      u8              DataSize,
      char           *ConversionName
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcEventCreate(
      mcTypeTaskRef ECURefNum,
      cstr          EventChannelName,
      u8            EventChannelNumber
      )
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcECUCreate(
      mcTypeTaskRef    DBRefNum,
      cstr             ECUName,
      char            *Interface,
      i32              ByteOrder,
      u32              CRO_ID,
      u32              DTO_ID,
      u16              StationAddress,
      u32              BaudRate,
      mcTypeTaskRef   *ECURefNum
      )
VX_EXPORT_SYMBOL;


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcConversionCreate(
      mcTypeTaskRef ECURefNum,
      char         *ConversionName,
      f64           Factor,
      f64           Offset,
      char         *Unit
      )
VX_EXPORT_SYMBOL;



/************************************************************************/
/* Some helper functions                                                */
/************************************************************************/
MS_EXPORT_SYMBOL
   void          CALLING_CONVENTION mcStatusToString(
      mcTypeStatus        Status,
      u32                 SizeofString,
      str                 ErrorString
      )
VX_EXPORT_SYMBOL;



MS_EXPORT_SYMBOL
  void CALLING_CONVENTION mcDebug(
     i32 a_Activate)
VX_EXPORT_SYMBOL;

// Begin Generic A2L Interface
// ---------------------------
MS_EXPORT_SYMBOL
   mcTypeA2LNode CALLING_CONVENTION mcA2LGetRootNode(
      mcTypeTaskRef DBRefNum)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeA2LNode CALLING_CONVENTION mcA2LGetParentNode(
      mcTypeTaskRef DBRefNum,
      mcTypeA2LNode hNode)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeA2LNode CALLING_CONVENTION mcA2LGetChildNodeByDescription(
      mcTypeTaskRef DBRefNum,
      mcTypeA2LNode hNode,
      const char *szDescription)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeA2LNode CALLING_CONVENTION mcA2LGetChildNodeByIndex(
      mcTypeTaskRef DBRefNum,
      mcTypeA2LNode hNode,
      u32 dwIdx)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeA2LNode CALLING_CONVENTION mcA2LGetChildTokenNode(
      mcTypeTaskRef DBRefNum,
      mcTypeA2LNode hNode,
      const char *szParamName,
      u32 dwParamIdx,
      u32 dwTokenIdx)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeA2LNode CALLING_CONVENTION mcA2LGetChildBlockNodeByIndex(
      mcTypeTaskRef DBRefNum,
      mcTypeA2LNode hNode,
      const char *szBlockName,
      u32 dwBlockIdx)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeA2LNode CALLING_CONVENTION mcA2LGetChildBlockNodeByIdentifier(
      mcTypeTaskRef DBRefNum,
      mcTypeA2LNode hNode,
      const char *szBlockName,
      const char *szBlockId)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeA2LNode CALLING_CONVENTION mcA2LGetSuccessorNodeByIndex(
      mcTypeTaskRef DBRefNum,
      mcTypeA2LNode hNode,
      u32 dwIdx)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   mcTypeA2LNode CALLING_CONVENTION mcA2LGetSuccessorNodeByNameAndIndex(
      mcTypeTaskRef DBRefNum,
      mcTypeA2LNode hNode,
      const char *szParamName,
      u32 dwIdx)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   TmcA2LNodeType CALLING_CONVENTION mcA2LGetNodeType(
      mcTypeTaskRef DBRefNum,
      mcTypeA2LNode hNode)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   const char * CALLING_CONVENTION mcA2LGetText(
      mcTypeTaskRef DBRefNum,
      mcTypeA2LNode hNode)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
  mcTypeStatus CALLING_CONVENTION mcA2LGetU32(
     mcTypeTaskRef DBRefNum,
     mcTypeA2LNode hNode,
     u32 *pValue)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
  mcTypeStatus CALLING_CONVENTION mcA2LGetI32(
     mcTypeTaskRef DBRefNum,
     mcTypeA2LNode hNode,
     i32 *pValue)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
  mcTypeStatus CALLING_CONVENTION mcA2LGetDouble(
     mcTypeTaskRef DBRefNum,
     mcTypeA2LNode hNode,
     double *pValue)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
  u32 CALLING_CONVENTION mcA2LGetBlockCount(
     mcTypeTaskRef DBRefNum,
     mcTypeA2LNode hNode,
     const char *szBlockName)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
  mcTypeStatus CALLING_CONVENTION mcA2LInitializeBlockArray(
     mcTypeTaskRef DBRefNum,
     mcTypeA2LNode hNode,
     const char *szBlockName,
     mcTypeA2LNode *pNodeArray,
     u32 dwNodeArraySize)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
  u32 CALLING_CONVENTION mcA2LGetIdentifierCount(
     mcTypeTaskRef DBRefNum,
     mcTypeA2LNode hNode,
     const char *szName)
VX_EXPORT_SYMBOL;

// End Generic A2L Interface
// -------------------------

// Begin Generic A2L Interface (using 64-bit node arguments)
// ---------------------------------------------------------
MS_EXPORT_SYMBOL
   u64 CALLING_CONVENTION mc64A2LGetRootNode(
      mcTypeTaskRef DBRefNum)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   u64 CALLING_CONVENTION mc64A2LGetParentNode(
      mcTypeTaskRef DBRefNum,
      u64 hNode)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   u64 CALLING_CONVENTION mc64A2LGetChildNodeByDescription(
      mcTypeTaskRef DBRefNum,
      u64 hNode,
      const char *szDescription)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   u64 CALLING_CONVENTION mc64A2LGetChildNodeByIndex(
      mcTypeTaskRef DBRefNum,
      u64 hNode,
      u32 dwIdx)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   u64 CALLING_CONVENTION mc64A2LGetChildTokenNode(
      mcTypeTaskRef DBRefNum,
      u64 hNode,
      const char *szParamName,
      u32 dwParamIdx,
      u32 dwTokenIdx)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   u64 CALLING_CONVENTION mc64A2LGetChildBlockNodeByIndex(
      mcTypeTaskRef DBRefNum,
      u64 hNode,
      const char *szBlockName,
      u32 dwBlockIdx)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   u64 CALLING_CONVENTION mc64A2LGetChildBlockNodeByIdentifier(
      mcTypeTaskRef DBRefNum,
      u64 hNode,
      const char *szBlockName,
      const char *szBlockId)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   u64 CALLING_CONVENTION mc64A2LGetSuccessorNodeByIndex(
      mcTypeTaskRef DBRefNum,
      u64 hNode,
      u32 dwIdx)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   u64 CALLING_CONVENTION mc64A2LGetSuccessorNodeByNameAndIndex(
      mcTypeTaskRef DBRefNum,
      u64 hNode,
      const char *szParamName,
      u32 dwIdx)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   TmcA2LNodeType CALLING_CONVENTION mc64A2LGetNodeType(
      mcTypeTaskRef DBRefNum,
      u64 hNode)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
   const char * CALLING_CONVENTION mc64A2LGetText(
      mcTypeTaskRef DBRefNum,
      u64 hNode)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
  mcTypeStatus CALLING_CONVENTION mc64A2LGetU32(
     mcTypeTaskRef DBRefNum,
     u64 hNode,
     u32 *pValue)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
  mcTypeStatus CALLING_CONVENTION mc64A2LGetI32(
     mcTypeTaskRef DBRefNum,
     u64 hNode,
     i32 *pValue)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
  mcTypeStatus CALLING_CONVENTION mc64A2LGetDouble(
     mcTypeTaskRef DBRefNum,
     u64 hNode,
     double *pValue)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
  u32 CALLING_CONVENTION mc64A2LGetBlockCount(
     mcTypeTaskRef DBRefNum,
     u64 hNode,
     const char *szBlockName)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
  mcTypeStatus CALLING_CONVENTION mc64A2LInitializeBlockArray(
     mcTypeTaskRef DBRefNum,
     u64 hNode,
     const char *szBlockName,
     u64 *pNodeArray,
     u32 dwNodeArraySize)
VX_EXPORT_SYMBOL;

MS_EXPORT_SYMBOL
  u32 CALLING_CONVENTION mc64A2LGetIdentifierCount(
     mcTypeTaskRef DBRefNum,
     u64 hNode,
     const char *szName)
VX_EXPORT_SYMBOL;
// End Generic A2L Interface (using 64-bit node arguments)
// -------------------------------------------------------


MS_EXPORT_SYMBOL
   mcTypeStatus CALLING_CONVENTION mcGetPropertyDataType(
      u32               PropertyID,
      tPropertyDataType *Value
      )
VX_EXPORT_SYMBOL;




#ifdef __cplusplus
   }
#endif

#endif // __niemc_h__
