/*--------------------------------------------------------------------------*/
/* Example: Action.c                                                        */
/* Purpose: execute an action service on ECUMC Demo ECU CCP                 */
/*    Note: To run this example, first start ECUMC Demo ECU CCP.exe from    */
/*          the ECUMC Demo ECU CCP example folder. This example will NOT    */
/*          work with the ECUMC Demo ECU XCP.                                */
/*          Since this example executes a shutdown on the ECU as an         */
/*          example, a communication failure will occur afterwards at       */
/*          the Disconnect. This is expected and does not indicate a        */
/*          failure of the ECU Measurement and Calibration Toolkit.         */
/*--------------------------------------------------------------------------*/

#define _CRT_SECURE_NO_WARNINGS
#define _CRT_NONSTDC_NO_DEPRECATE

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <niemc.h>

#include "action.h"

#define CFALSE   0
#define CTRUE    1

#ifdef LINUX_BUILD
   #define A2LFILE      "../a2l/CCPECU.a2l"
#else
    #define A2LFILE      "..\\a2l\\CCPECU.a2l"
#endif

#define MYINTERFACE  "CCP:CAN1"
#define XNET         CTRUE   // Enable NI-XNET or NI-CAN, TRUE = NI-XNET , FALSE = NI-CAN
#define SHUTDOWN     1


mcTypeTaskRef DBRef = 0, ECURef = 0;


/*--------------------------------------------------------------------------*/
/*   Function:   CheckError                                                 */
/*    Purpose:   display a message in case of an ECUMC error and terminate  */
/*--------------------------------------------------------------------------*/
void CheckError (mcTypeStatus Stat, char *Function)
{
   char szErrText[1024], szTitle[128];

   if (Stat == 0)
      return;

   mcStatusToString (Stat, sizeof (szErrText), szErrText);

   sprintf (szTitle, "%s in function %s", Stat & 0x80000000 ? "Error" : "Warning", Function);
   printf ("%s:\n%s\n", szTitle, szErrText);

   if (Stat > 0)
      return;

   printf ("Press RETURN to continue...\n");
   getchar ();

   if (ECURef != 0)
   {
      mcECUDeselect (&ECURef);
      ECURef = 0;
   }

   if (DBRef != 0)
   {
      mcDatabaseClose (&DBRef);
      DBRef = 0;
   }

   exit (0);
}


/*--------------------------------------------------------------------------*/
/*   Function:   main                                                       */
/*    Purpose:   the main function for this example                         */
/*--------------------------------------------------------------------------*/
int main (void)
{
   mcTypeStatus Stat;
   u8 Params[] = { 0, 0, 0, 0 };
   u8 nLength, nDataType;
   char Interface[20];

   strcpy(Interface, MYINTERFACE);
   if(XNET)
   {
      // NI-XNET enabled
      strcat (Interface, "@nixnet");
   }

   printf ("Opening %s...\n", A2LFILE);
   Stat = mcDatabaseOpen (A2LFILE, &DBRef);
   CheckError (Stat, "mcDatabaseOpen");
   Stat = mcECUSelectEx (DBRef, NULL, Interface, &ECURef);
   CheckError (Stat, "mcECUSelectEx");
   Stat = mcECUConnect (ECURef);
   CheckError (Stat, "mcECUConnect");

   printf ("Executing SHUTDOWN action...\n");
   Stat = mcCCPActionService (ECURef, SHUTDOWN, Params, &nLength, &nDataType);
   CheckError (Stat, "mcCCPActionService");

   Stat = mcECUDisconnect (ECURef);
   CheckError (Stat, "mcECUDisconnect");
   Stat = mcECUDeselect (&ECURef);
   CheckError (Stat, "mcECUDeselect");
   ECURef = 0;
   Stat = mcDatabaseClose (&DBRef);
   CheckError (Stat, "mcDatabaseClose");
   DBRef = 0;

   printf ("Action executed...\nPress RETURN to continue...");
   getchar ();

   return 0;
}
